#include "../tools/tools.h"

#include <EGrabber.h>
#include <RGBConverter.h>

using namespace Euresys;

static void sample() {
    GenTL genTL; // load Coaxlink CTI
    EGrabber<CallbackOnDemand> grabber(genTL); // create grabber
    RGBConverter rgbConverter(genTL); // create rgb converter environment

    // configure grabber using script file 100-grabn.setup.js
    grabber.runScript(Tools::getScriptPath("setup"));
    grabber.reallocBuffers(20); // prepare 20 buffers

    grabber.start(20); // grab 20 buffers
    for (size_t frame = 0; frame < 20; ++frame) {
        ScopedBuffer buffer(grabber); // wait and get a buffer
        // Note: ScopedBuffer pushes the buffer back to the input queue automatically
        uint8_t *imagePointer = buffer.getInfo<uint8_t *>(gc::BUFFER_INFO_BASE);
        // get the raw buffer image pointer and pass it to a BGR8 converter
        RGBConverter::BGR8 bgr(rgbConverter, imagePointer,
            buffer.getInfo<uint64_t>(gc::BUFFER_INFO_PIXELFORMAT),
            buffer.getInfo<size_t>(gc::BUFFER_INFO_WIDTH),
            buffer.getInfo<size_t>(gc::BUFFER_INFO_DELIVERED_IMAGEHEIGHT));
        // output the converted buffer
        bgr.saveToDisk(Tools::getEnv("sample-output-path") + "/frame.NNN.jpeg", frame);
    }
}

static Tools::Sample addSample(__FILE__, "Simple Grab N frames", sample);
